@@ -9,7 +9,7 @@ OPTKNOCK = 'optknock'
 ROBUSTKNOCK = 'robustknock'
 OPTCOUPLE = 'optcouple'
 
-TIME_LIMIT_W_SOL   = TIME_LIMIT+'_w_sols'
+TIME_LIMIT_W_SOL = TIME_LIMIT + '_w_sols'
 ERROR = 'error'
 
 KOCOST = 'ko_cost'
@@ -40,8 +40,8 @@ GLPK = 'glpk'
 SETUP = 'sd_setup'
 MAX_SOLUTIONS = 'max_solutions'
 MAX_COST = 'max_cost'
-TIME_LIMIT = 'time_limit'
+T_LIMIT = 'time_limit'
 SOLUTION_APPROACH = 'solution_approach'
 ANY = 'any'
 BEST = 'best'
-POPULATE = 'populate'
\ No newline at end of file
+POPULATE = 'populate'
